/* 
 * titrax.c
 *
 * A simple titrax clone written from scratch.
 * ("simple" nowadays is a good joke)
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * Caution: The ID's for the edit-buttons in titrax.h are critical! The
 *          program will calculate the amount of time from the value of
 *          this ID's.
 *
 * $Id: titrax.c,v 1.18 1997/11/23 20:08:59 ga58 Exp ga58 $
 *
 * v1.0 25.09.1997 be
 * v1.1 03.10.1997 be found an overflow condition in TimeToString, code 
 *                    clean-up.
 * v1.2 11.10.1997 be changed edit-form layout
 * v1.3 12.10.1997 be introduced a new layout for the change-time part of 
 *                    the edit-form
 * v1.4 17.10.1997 be test-only version
 *
 * $Log: titrax.c,v $
 * Revision 1.18  1997/11/23 20:08:59  ga58
 * - added an experimental timestamp option
 *
 * Revision 1.17  1997/11/18 19:49:06  ga58
 * - menues can be invoked through the windows title bar
 *
 * Revision 1.16  1997/11/17 21:35:25  ga58
 * - fixed the redraw bug in the notes-dialog
 * - added undo and select all menu options in the notes and edit dialog
 *
 * Revision 1.15  1997/11/16 17:42:58  ga58
 * - removed the scrolling glitch in the note-dialog
 * - new about box
 *
 * Revision 1.14  1997/11/15 17:59:29  ga58
 * *** empty log message ***
 *
 * Revision 1.13  1997/11/14 17:49:10  ga58
 * - seperated the sources into various files
 * - switching between projects with the up and down buttons
 * - removed timestamps
 * - added scrolling for the notes window
 * - interface changes
 *
 * Revision 1.12  1997/11/12 17:42:47  ga58
 * - changed the note icons
 * - added a timestamp menu option
 * - added a timestamp button in the note dialog
 *
 * Revision 1.11  1997/11/09 14:59:18  ga58
 * - added a justification toggle for the project names
 * - all times are displayed right justified
 * - added cut, copy and paste menu items to the edit and note dialogs
 * - added a separate clear all button
 * - tapping in the time area invokes the edit dialog
 * - cleaned up and improved the tip-menu
 *
 * Revision 1.10  1997/11/08 18:24:18  ga58
 * - new layout
 * - pushbuttons for the labels
 * - new redraw code
 *
 * Revision 1.9  1997/11/06 18:10:49  ga58
 * - added utility macro MakeDigits
 * - EditViewInit and NoteViewInit only set the selection, if the string is empt
 *
 * Revision 1.8  1997/11/02 17:32:48  ga58
 * - titrax can now save all data in a memopad note
 *
 * Revision 1.7  1997/11/02 15:23:34  ga58
 * - finished the switch to a databased application
 * - some functionality added
 * - code cleanup
 *
 * Revision 1.6  1997/10/26 17:35:08  ga58
 * complete rewrite, now uses database for the slots
 *
 *
 */

#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <System/Graffiti.h>
#include <UI/UIAll.h>
#include <UI/UIGlobals.h>

#include "titraxAll.h"

/*
 * Time measuring concept: when a project is started, the variable
 * startedAt is filled with the current time (TimGetSeconds). At each 
 * checkpoint the difference between startedAt and the current time is 
 * added to the variable elpased and startedAt is filled with the 
 * current time. Only the currently active project has running == true.
 */

/*
 * It should be possible to get rid of this variable (used for
 * LstSetListChoices). But how???
 */
Char entry[MAXSLOT][MAXTEXT+1];
CharPtr entryP[MAXSLOT];

/*
 * procedure declarations
 */
static Boolean StartApplication(void);
static void StopApplication(void);

static void EventLoop(void);
static Boolean ApplicationHandleEvent(EventPtr event);
static Boolean EditViewHandleEvent(EventPtr event);
static Boolean NoteViewHandleEvent(EventPtr event);
static Boolean MainViewHandleEvent(EventPtr event);

static void EditViewInit(void);
static void NoteViewInit(void);
static void MainViewInit(void);

static void ClearSlot(Int no);
static void ClearAllSlots(void);
static void ClearTime(Int no);
static void ClearAllTimes(void);

static Int ActiveSlot(void);
static void ActivateSlot(Int no, Boolean update);
static void DeActivateSlot(Int no, Boolean update);
static void UpdateSlot(Int no, Boolean button, Boolean note);
static void UpdateAllSlots(Boolean button, Boolean note);

static void DrawButton(Int no);
static void DrawNoteButton(Int no);
static void DrawSumTimeButton(void);

static void NoteViewCheckScroll(void);

/*
 * DrawXXXX-routines
 *
 * All the routines must be called exclusively from the XXXXSlot-routines
 */
static void
DrawSumTimeButton(void)
{
  Int i;
  Int cLen;
  Int sLen;
  Char space[2*MAXTEXT];
  ULong t = 0;
  Char text[10];

  for(i = 0; i < MAXSLOT; i++)
	 t += GetElapsed(i);

  StrCopy(space, "");
  StrCopy(text, TimeToString(t, text));

  cLen = FntCharsWidth(text, StrLen(text));
  sLen = FntCharWidth(' ');

  for(i = cLen; i < 35; i += sLen)
	 StrCat(space, " ");
  StrCat(space, text);

  CtlSetLabel(GetObjectPtr(timeSum), space);
}

static void
DrawButton(Int no)
{
  Int i;
  Int cLen;
  Int sLen;
  Int tLen;

  Char text[4*MAXTEXT+1];
  Char time[10];

  const Int width = BUTTONW - 2;

  GetText(no, text);

  if (TextIsEmpty(no) == false) {
	 TimeToString(GetElapsed(no), time);
	 tLen = FntCharsWidth(time, StrLen(time));
	 cLen = FntCharsWidth(text, StrLen(text));
	 sLen = FntCharWidth(' ');
	 
	 for(i = cLen + tLen; i < width; i += sLen)
		StrCat(text, " ");
	 StrCat(text, time);
  }
  
  CtlSetLabel(GetObjectPtr(label0 + no), text);
}

static void
DrawNoteButton(Int no)
{
  ControlPtr controlP;
  
  controlP = GetObjectPtr(note0 + no);

  if (TextIsEmpty(no)) {
	 CtlHideControl(controlP);
  }
  else {
	 CtlShowControl(controlP);
	 if (NoteIsEmpty(no))
		CtlSetLabel(controlP, "\013");
	 else
		CtlSetLabel(controlP, "\011");
  }
}

/*
 * XXXXSlot-routines
 *
 * All changes to the global control-structures MUST use this routines
 */
static void
DeActivateSlot(Int no, Boolean update)
{
  if (no == noSlotActive)
	 return;

  if (IsRunning(no) == true) {
	 SetElapsed(no, GetElapsed(no) + TimGetSeconds() - GetStartedAt(no));
	 SetRunning(no, false);
	 SetStartedAt(no, 0);

	 if (Track(no))
		AddTimeStamp(no, false);
  }

  if (update) {
	 UpdateSlot(no, true, Track(no));
	 DrawSumTimeButton();
  }

}

static void
ActivateSlot(Int no, Boolean update)
{
  DeActivateSlot(ActiveSlot(), update);

  if (IsRunning(no) == false) {
	 SetRunning(no, true);
	 SetStartedAt(no, TimGetSeconds());
	 
	 if (Track(no))
		AddTimeStamp(no, true);
  }

  if (update) {
	 UpdateSlot(no, true, Track(no));
	 DrawSumTimeButton();
  }
}

static void
UpdateSlot(Int no, Boolean button, Boolean note)
{
  ULong t;

  if (no == noSlotActive)
	 return;

  if (no == ActiveSlot()) {
	 t = TimGetSeconds();
	 SetElapsed(no, GetElapsed(no) + t - GetStartedAt(no));
	 SetStartedAt(no, t);
  }

  if (button)
	 DrawButton(no);
  if (note)
	 DrawNoteButton(no);

  if (IsRunning(no))
	 CtlSetValue(GetObjectPtr(label0 + no), 1);
  else
	 CtlSetValue(GetObjectPtr(label0 + no), 0);
}

static void
UpdateAllSlots(Boolean button, Boolean note)
{
  int i;

  for(i = 0; i < MAXSLOT; i++)
	 UpdateSlot(i, button, note);
}

static Int
ActiveSlot(void)
{
  int i;
  
  for(i = 0; i < MAXSLOT; i++)
	 if (IsRunning(i) == true)
		return i;

  return noSlotActive;
}

/*
 * ClearXXXX-routines
 */
static void
ClearTime(Int no)
{
  if (no == noSlotActive)
	 return;

  SetRunning(no, false);
  SetElapsed(no, 0);
  SetStartedAt(no, 0);

  ClearNote(no);
}

static void
ClearSlot(Int no)
{
  if (no == noSlotActive)
	 return;

  ClearTime(no);

  SetText(no, "");
  SetTrack(no, false);
}

static void
ClearAllTimes(void)
{
  int i;
  
  for(i = 0; i < MAXSLOT; i++)
	 ClearTime(i);
}

static void
ClearAllSlots(void)
{
  int i;

  for(i = 0; i < MAXSLOT; i++)
	 ClearSlot(i);
}

/*
 * Helper function: convert the time into 5 digits
 */
static void
DrawDigits(ULong t)
{
  Char s1[2], s2[2], s3[2], s4[2], s5[2], s6[2];

  StrCopy(s1, StrIToA(s1, t / tendays));
  t %= tendays;
  StrCopy(s2, StrIToA(s2, t / oneday));
  t %= oneday;
  StrCopy(s3, StrIToA(s3, t / tenhours));
  t %= tenhours;
  StrCopy(s4, StrIToA(s4, t / onehour));
  t %= onehour;
  StrCopy(s5, StrIToA(s5, t / tenmins));
  t %= tenmins;
  StrCopy(s6, StrIToA(s6, t / onemin));

  CtlSetLabel(GetObjectPtr(editDayHigh), s1);
  CtlSetLabel(GetObjectPtr(editDayLow), s2);
  CtlSetLabel(GetObjectPtr(editHourHigh), s3);
  CtlSetLabel(GetObjectPtr(editHourLow), s4);
  CtlSetLabel(GetObjectPtr(editMinHigh), s5);
  CtlSetLabel(GetObjectPtr(editMinLow), s6);
}

/*
 * XXXXViewInit
 *
 * form-initialization
 */
static void 
EditViewInit(void)
{
  Int i;
  Int j = 0;
  Int selectedShiftSlot;
  FormPtr frm;
  ListPtr listP;
  FieldPtr fldP;
  ControlPtr controlP;
  Char text[MAXTEXT+1];

  GetText(EditedSlot(), text);
  fldP = GetObjectPtr(editField);
  FldSetTextHandle(fldP, GetRecordHandle(TextRecIdx(EditedSlot())));

  frm = FrmGetActiveForm();
  FrmDrawForm(frm);

  FldSetInsPtPosition(fldP, StrLen(text));

  DrawDigits(GetElapsed(EditedSlot()));

  if (Increment() == onemin)
	 CtlSetValue(GetObjectPtr(editMinLow), 1);
  else if (Increment() == tenmins)
	 CtlSetValue(GetObjectPtr(editMinHigh), 1);
  else if (Increment() == onehour)
	 CtlSetValue(GetObjectPtr(editHourLow), 1);
  else if (Increment() == tenhours)
	 CtlSetValue(GetObjectPtr(editHourHigh), 1);
  else if (Increment() == oneday)
	 CtlSetValue(GetObjectPtr(editDayLow), 1);
  else if (Increment() == tendays)
	 CtlSetValue(GetObjectPtr(editDayHigh), 1);
  else {
	 SetIncrement(onemin);
	 CtlSetValue(GetObjectPtr(editMinLow), 1);
  }

  for(i = 0; i < MAXSLOT; i++) {
	 if (i == EditedSlot())
		continue;
	 if (TextIsEmpty(i) == false) {
		GetText(i, text);
		StrCopy(entry[j], text);
		entryP[j] = entry[j];
		SetShiftSlot(j, i);
		j++;
	 }
  }

  listP = GetObjectPtr(editShiftList);
  controlP = GetObjectPtr(editShift);

  if (j > 0) {
	 CtlShowControl(GetObjectPtr(editS5M));
	 CtlShowControl(GetObjectPtr(editS30M));
	 CtlShowControl(GetObjectPtr(editS60M));
	 LstSetListChoices(listP, entryP, j);
	 LstSetHeight(listP, j);
	 LstSetSelection(listP, 0);
	 CtlShowControl(GetObjectPtr(editShift));
	 CtlSetLabel(GetObjectPtr(editShift), LstGetSelectionText(listP, LstGetSelection(listP)));
	 CtlShowControl(GetObjectPtr(editShiftTime));
	 selectedShiftSlot = ShiftSlot(LstGetSelection(listP));
	 controlP = GetObjectPtr(editShiftTime);
	 CtlSetLabel(controlP, TimeToString(GetElapsed(selectedShiftSlot), text));
  }
  else {
	 CtlHideControl(GetObjectPtr(editS5M));
	 CtlHideControl(GetObjectPtr(editS30M));
	 CtlHideControl(GetObjectPtr(editS60M));
	 CtlHideControl(GetObjectPtr(editShift));
	 CtlHideControl(GetObjectPtr(editShiftTime));
  }

  CtlSetValue(GetObjectPtr(editTrack), Track(EditedSlot()));

  FrmSetFocus(frm, FrmGetObjectIndex(frm, editField));

  if (TextIsEmpty(EditedSlot()) == true)
	 GrfSetState(false, false, true);
}

static void
NoteViewInit(void)
{
  FormPtr frm;
  FieldPtr fldP;
  Int noteLen;
  Char textStr[MAXTEXT+1];
  Char title[MAXTEXT+12];

  frm = FrmGetActiveForm();
  FrmDrawForm(frm);

  GetText(EditedNoteSlot(), textStr);
  StrCopy(title, "Notes for ");
  StrCat(title, textStr);
  FrmCopyTitle(frm, title);


  noteLen = NoteLen(EditedNoteSlot());
  fldP = GetObjectPtr(noteField);
  FldSetTextHandle(fldP, GetRecordHandle(NoteRecIdx(EditedNoteSlot())));

  FldSetInsPtPosition(fldP, noteLen);

  FrmSetFocus(frm, FrmGetObjectIndex(frm, noteField));

  if (NoteIsEmpty(EditedNoteSlot()) == true)
	 GrfSetState(false, false, true);

  FldDrawField(GetObjectPtr(noteField));

  NoteViewCheckScroll();
}

static void
NoteViewCheckScroll()
{
  FormPtr frm;
  Boolean canUp, canDown;
  
  frm = FrmGetActiveForm();
  canUp = FldScrollable(GetObjectPtr(noteField), up);
  canDown = FldScrollable(GetObjectPtr(noteField), down);

  FrmUpdateScrollers(frm, 
							FrmGetObjectIndex(frm, noteUp), 
							FrmGetObjectIndex(frm, noteDown),
							canUp, canDown);
}

static void
MainViewInit(void)
{
  FormPtr frm;

  frm = FrmGetActiveForm();
  FrmDrawForm(frm);

  UpdateAllSlots(true, true);
  DrawSumTimeButton();
}

/*
 * Eventloops
 */
static void
EventLoop(void)
{
  EventType event;
  Word error;

  do {
	 EvtGetEvent(&event, 100);

	 if (!SysHandleEvent(&event))
		if (!MenuHandleEvent(0, &event, &error))
		  if (!ApplicationHandleEvent(&event))
			 FrmDispatchEvent(&event);
  } while (event.eType != appStopEvent);
}

static Boolean
ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent) {
	 formId = event->data.frmLoad.formID;
	 frm = FrmInitForm(formId);
	 FrmSetActiveForm(frm);

	 switch (formId) {
	 case mainForm:
		FrmSetEventHandler(frm, MainViewHandleEvent);
		break;
	 case editForm:
		FrmSetEventHandler(frm, EditViewHandleEvent);
		break;
	 case noteForm:
		FrmSetEventHandler(frm, NoteViewHandleEvent);
		break;
	 }
	 handled = true;
  }
  return handled;
}

static Boolean
NoteViewHandleEvent(EventPtr event)
{
  Char timStr[timeStringLength];
  Char datStr[dateStringLength];
  Boolean handled = false;
  MenuBarPtr menuP;

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == noteOk) {
		FldCompactText(GetObjectPtr(noteField));
		FldSetDirty(GetObjectPtr(noteField), true);
		FldSetTextHandle(GetObjectPtr(noteField), 0);
		ReleaseRecordHandle(NoteRecIdx(EditedNoteSlot()), true);
		if (NoteIsEmpty(EditedNoteSlot()) == true)
		  ClearNote(EditedNoteSlot());
		FrmReturnToForm(mainForm);
		UpdateSlot(EditedNoteSlot(), false, true);
		UpdateSlot(ActiveSlot(), true, false);
		DrawSumTimeButton();
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == noteTimeStamp) {
		TimeString(timStr);
		FldInsert(GetObjectPtr(noteField), timStr, StrLen(timStr));
		NoteViewCheckScroll();
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == noteDateStamp) {
		DateString(datStr);
		FldInsert(GetObjectPtr(noteField), datStr, StrLen(datStr));
		NoteViewCheckScroll();
		handled = true;
	 }
	 break;

  case ctlRepeatEvent:
	 if (event->data.ctlRepeat.controlID == noteUp) {
		if (FldScrollable(GetObjectPtr(noteField), up))
		  FldScrollField(GetObjectPtr(noteField), 1, up);

		NoteViewCheckScroll();
		handled = false;
	 }
	 else if (event->data.ctlRepeat.controlID == noteDown) {
		if (FldScrollable(GetObjectPtr(noteField), down))
		  FldScrollField(GetObjectPtr(noteField), 1, down);
		
		NoteViewCheckScroll();
		handled = false;
	 }
	 break;

  case keyDownEvent:
	 if (event->data.keyDown.chr == pageUpChr) {
		if (FldScrollable(GetObjectPtr(noteField), up))
		  FldScrollField(GetObjectPtr(noteField), FldGetVisibleLines(GetObjectPtr(noteField))-1, up);
		NoteViewCheckScroll();
		handled = true;
	 }
	 else if (event->data.keyDown.chr == pageDownChr) {
		if (FldScrollable(GetObjectPtr(noteField), down))
		  FldScrollField(GetObjectPtr(noteField), FldGetVisibleLines(GetObjectPtr(noteField))-1, down);
		NoteViewCheckScroll();
		handled = true;
	 }
	 else {
		handled = FrmHandleEvent(FrmGetActiveForm(), event);
		NoteViewCheckScroll();
	 }
	 break;

  case frmOpenEvent:
	 NoteViewInit();
	 handled = true;
	 break;

  case appStopEvent:
	 FldSetDirty(GetObjectPtr(noteField), true);
	 FldSetTextHandle(GetObjectPtr(noteField), 0);
	 ReleaseRecordHandle(NoteRecIdx(EditedNoteSlot()), true);
	 if (NoteIsEmpty(EditedNoteSlot()) == true)
		ClearNote(EditedNoteSlot());
	 SetCurrentView(noteForm);
	 handled = false;
	 break;

  case frmTitleSelectEvent:
	 menuP = MenuInit(titraxNoteMenu);
	 MenuSetActiveMenu(menuP);
	 MenuDrawMenu(menuP);
	 handled = true;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == titraxNoteMenuKeyboard)
		SysKeyboardDialogV10();
	 if (event->data.menu.itemID == titraxNoteMenuGraffiti)
		SysGraffitiReferenceDialog(referenceDefault);
	 if (event->data.menu.itemID == titraxNoteMenuCut)
		FldCut(GetObjectPtr(noteField));
	 if (event->data.menu.itemID == titraxNoteMenuCopy)
		FldCopy(GetObjectPtr(noteField));
	 if (event->data.menu.itemID == titraxNoteMenuPaste)
		FldPaste(GetObjectPtr(noteField));
	 if (event->data.menu.itemID == titraxNoteMenuUndo)
		FldUndo(GetObjectPtr(noteField));
	 if (event->data.menu.itemID == titraxNoteMenuSelectAll)
		FldSetSelection(GetObjectPtr(noteField), 0, 
							 FldGetTextLength(GetObjectPtr(noteField)));
	 NoteViewCheckScroll();

	 handled = true;
	 break;

  default:
	 handled = false;
  }
  return handled;
}

static Boolean
EditViewHandleEvent(EventPtr event)
{
  Char text[MAXTEXT+1];
  ListPtr listP;
  Boolean handled = false;
  Int selectedShiftSlot;
  Int amount;
  MenuBarPtr menuP;

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == editOK) {
		FldSetDirty(GetObjectPtr(editField), true);
		FldSetTextHandle(GetObjectPtr(editField), 0);
		ReleaseRecordHandle(TextRecIdx(EditedSlot()), true);
		FrmReturnToForm(mainForm);

		if (TextIsEmpty(EditedSlot()) == true) {
		  ClearSlot(EditedSlot());
		  DeActivateSlot(EditedSlot(), false);
		}
		else {
		  ActivateSlot(EditedSlot(), false);
		}
		UpdateSlot(EditedSlot(), false, true);
		UpdateAllSlots(true, false);
		DrawSumTimeButton();
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == editTrack) {
		SetTrack(EditedSlot(), CtlGetValue(GetObjectPtr(editTrack)));
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == editDayHigh) {
		SetIncrement(tendays);
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == editDayLow) {
		SetIncrement(oneday);
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == editHourHigh) {
		SetIncrement(tenhours);
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == editHourLow) {
		SetIncrement(onehour);
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == editMinHigh) {
		SetIncrement(tenmins);
		handled = true;
	 }
	 if (event->data.ctlEnter.controlID == editMinLow) {
		SetIncrement(onemin);
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == editUp) {
		SetElapsed(EditedSlot(), GetElapsed(EditedSlot()) + Increment());
		DrawDigits(GetElapsed(EditedSlot()));
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == editDown) {
		if (GetElapsed(EditedSlot()) >= Increment())
		  SetElapsed(EditedSlot(), GetElapsed(EditedSlot()) - Increment());
		else
		  SetElapsed(EditedSlot(), 0);

		DrawDigits(GetElapsed(EditedSlot()));
		handled = true;
	 }

	 if ((event->data.ctlEnter.controlID >= editS5M) && (event->data.ctlEnter.controlID <= editS60M)) {
		amount = (event->data.ctlEnter.controlID - editS0M) * 5 * 60;
		listP = GetObjectPtr(editShiftList);
		if (LstGetSelection(listP) != noListSelection) {
		  selectedShiftSlot = ShiftSlot(LstGetSelection(listP));
		  if (GetElapsed(EditedSlot()) > amount) {
			 SetElapsed(EditedSlot(), GetElapsed(EditedSlot()) - amount);
			 SetElapsed(selectedShiftSlot, GetElapsed(selectedShiftSlot) + amount);
		  }
		  else {
			 SetElapsed(selectedShiftSlot, GetElapsed(selectedShiftSlot) + GetElapsed(EditedSlot()));
			 SetElapsed(EditedSlot(), 0);
		  }

		  DrawDigits(GetElapsed(EditedSlot()));

		  CtlSetLabel(GetObjectPtr(editShiftTime), TimeToString(GetElapsed(selectedShiftSlot), text));
		}
		handled = true;
	 }
	 break;

  case popSelectEvent:
	 listP = GetObjectPtr(editShiftList);
	 if (LstGetSelection(listP) != noListSelection) {
		selectedShiftSlot = ShiftSlot(LstGetSelection(listP));
		CtlSetLabel(GetObjectPtr(editShiftTime), TimeToString(GetElapsed(selectedShiftSlot), text));
	 }
	 handled = false; /* so the OS will redraw the button for us */
	 break;
	 
  case frmOpenEvent:
	 EditViewInit();
	 handled = true;
	 break;

  case appStopEvent:
	 FldSetDirty(GetObjectPtr(editField), true);
	 FldSetTextHandle(GetObjectPtr(editField), 0);
	 ReleaseRecordHandle(TextRecIdx(EditedSlot()), true);
	 if (TextIsEmpty(EditedSlot()) == true) {
		ClearSlot(EditedSlot());
	 }
	 SetCurrentView(editForm);
	 handled = false;
	 break;
	 
  case frmTitleSelectEvent:
	 menuP = MenuInit(titraxEditMenu);
	 MenuSetActiveMenu(menuP);
	 MenuDrawMenu(menuP);
	 handled = true;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == titraxEditMenuKeyboard)
		SysKeyboardDialogV10();
	 if (event->data.menu.itemID == titraxEditMenuGraffiti)
		SysGraffitiReferenceDialog(referenceDefault);
	 if (event->data.menu.itemID == titraxEditMenuCut)
		FldCut(GetObjectPtr(editField));
	 if (event->data.menu.itemID == titraxEditMenuCopy)
		FldCopy(GetObjectPtr(editField));
	 if (event->data.menu.itemID == titraxEditMenuPaste)
		FldPaste(GetObjectPtr(editField));
	 if (event->data.menu.itemID == titraxEditMenuUndo)
		FldUndo(GetObjectPtr(editField));
	 if (event->data.menu.itemID == titraxEditMenuSelectAll)
		FldSetSelection(GetObjectPtr(editField), 0, 
							 FldGetTextLength(GetObjectPtr(editField)));
	 handled = true;
	 break;

  default:
	 handled = false;
  }
  return handled;
}

static Boolean
MainViewHandleEvent(EventPtr event)
{
  Boolean handled = false;
  Int cid;
  Int slotNo = noSlotActive;
  Int nextSlot = noSlotActive;
  FormPtr frm;
  MenuBarPtr menuP;

  switch (event->eType) {
  case ctlSelectEvent:
	 cid = event->data.ctlEnter.controlID;
	 if (cid == mainEditButton) {
		if (ActiveSlot() != noSlotActive) {
		  SetEditedSlot(ActiveSlot());
		  FrmPopupForm(editForm);
		}
		else {
		  (void)FrmAlert(titraxEditButtonAlert);
		}
		handled = true;
	 }

	 if (cid == mainClearButton) {
		if (ActiveSlot() != noSlotActive) {
		  if (FrmAlert(titraxClearAlert) == 0) {
			 slotNo = ActiveSlot();
			 ClearTime(slotNo);
			 UpdateSlot(slotNo, true, true);
			 UpdateSlot(ActiveSlot(), true, false);
			 DrawSumTimeButton();
		  }
		}
		else {
		  (void)FrmAlert(titraxClearButtonAlert);
		}
		handled = true;
	 }

	 if (cid == mainClearAllButton) {
		if (FrmAlert(titraxClearAllAlert) == 0) {
		  ClearAllTimes();
		  UpdateAllSlots(true, true);
		  DrawSumTimeButton();
		}
		handled = true;
	 }

	 if (cid == mainSaveAllButton) {
		if (FrmAlert(titraxSaveAllAlert) == 0) {
		  UpdateSlot(ActiveSlot(), true, false);
		  DrawSumTimeButton();
		  SaveData();
		}
		if (FrmAlert(titraxClearAllAlert) == 0) {
		  ClearAllTimes();
		  UpdateAllSlots(true, true);
		  DrawSumTimeButton();
		}
		handled = true;
	 }
	 
	 if ((cid >= label0) && (cid < label0 + MAXSLOT)) {
		slotNo = cid - label0;

		if (IsRunning(slotNo) == true) {
		  DeActivateSlot(slotNo, true);
		}
		else {
		  if (TextIsEmpty(slotNo) == true) {
			 SetEditedSlot(slotNo);
			 FrmPopupForm(editForm);
		  }
		  else {
			 ActivateSlot(slotNo, true);
		  }
		}
		handled = true;
	 }

	 if ((cid >= note0) && (cid < note0 + MAXSLOT)) {
		SetEditedNoteSlot(cid - note0);
		FrmPopupForm(noteForm);
		handled = true;
	 }

	 break;

  case frmOpenEvent:
	 MainViewInit();
	 handled = true;
	 break;

  case frmUpdateEvent:
	 UpdateAllSlots(true, true);
	 DrawSumTimeButton();
	 handled = true;
	 break;

  case appStopEvent:
	 SetCurrentView(mainForm);
	 handled = false;
	 break;

  case nilEvent:
	 if ((TimGetSeconds() - LastCall()) > 60) {
		SetLastCall(TimGetSeconds());
		UpdateSlot(ActiveSlot(), true, false);
		DrawSumTimeButton();
	 }
	 handled = true;
	 break;

  case keyDownEvent:
	 if (event->data.keyDown.chr == pageUpChr) {
		slotNo = ActiveSlot();
		if (slotNo != noSlotActive) {
		  nextSlot = (slotNo - 1);
		  if (nextSlot < 0)
			 nextSlot = MAXSLOT - 1;
		  while (TextIsEmpty(nextSlot)) {
			 nextSlot--;
			 if (nextSlot < 0)
				nextSlot = MAXSLOT - 1;
		  }
		  DeActivateSlot(slotNo, true);
		  ActivateSlot(nextSlot, true);
		}
	 }

	 if (event->data.keyDown.chr == pageDownChr) {
		slotNo = ActiveSlot();
		if (slotNo != noSlotActive) {
		  nextSlot = (slotNo + 1) % MAXSLOT;
		  while (TextIsEmpty(nextSlot))
			 nextSlot = (nextSlot + 1) % MAXSLOT;
		  DeActivateSlot(slotNo, true);
		  ActivateSlot(nextSlot, true);
		}
	 }
	 handled = true;
	 break;

  case frmTitleSelectEvent:
	 menuP = MenuInit(titraxMainMenu);
	 MenuSetActiveMenu(menuP);
	 MenuDrawMenu(menuP);
	 handled = true;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == titraxMainMenuAbout) {
		frm = FrmInitForm(aboutForm);
		FrmDoDialog(frm);
		FrmDeleteForm(frm);
	 }
	 handled = true;
	 break;

  default:
	 handled = false;
  }

  return handled;
}

/*
 * Init-routine
 */
static Boolean
StartApplication(void)
{
  CreatePrefs();

  if (!ReadPrefs()) {
	 ClearPrefs();
	 CreateDatabase();
	 OpenDatabase();
	 CreateRecords();
	 ClearAllSlots(); /* total reset */
  }
  else {
	 OpenDatabase();
  }
  return false;
}

/*
 * Exit-routine
 */
static void
StopApplication(void)
{
  FrmSaveAllForms();
  FrmCloseAllForms();
 
  WritePrefs();
  DeletePrefs();

  CloseDatabase();
}

/*
 * Main
 */
DWord 
PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
 if (cmd == sysAppLaunchCmdNormalLaunch) {
	if (!StartApplication()) {
	  FrmGotoForm(mainForm);
	  /* 
		* if we were interrupted while not in the main-form, restore the
		* last active form
		*/
	  if (CurrentView() != mainForm)
		 FrmGotoForm(CurrentView());
	  EventLoop();
	  StopApplication();
	}
 }
 return 0;
}
