/*
 * Titrax utility subroutines
 *
 * $Id: titraxUtil.c,v 1.5 1997/12/06 13:42:04 ga58 Exp ga58 $
 *
 * $Log: titraxUtil.c,v $
 * Revision 1.5  1997/12/06 13:42:04  ga58
 * - total time is no saved into the memopad note
 *
 * Revision 1.4  1997/11/23 20:08:59  ga58
 * *** empty log message ***
 *
 * Revision 1.3  1997/11/23 15:30:38  ga58
 * - fixed a off-by-one bug in ReadNote
 *
 * Revision 1.2  1997/11/15 17:59:29  ga58
 * *** empty log message ***
 *
 * Revision 1.1  1997/11/14 17:51:04  ga58
 * Initial revision
 *
 */

#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <System/Graffiti.h>
#include <UI/UIAll.h>
#include <UI/UIGlobals.h>

#include "titraxAll.h"

/*
 * GetObjectPtr helper-function
 */
VoidPtr
GetObjectPtr(Int ObjectID)
{
  FormPtr frm;

  frm = FrmGetActiveForm();
  return (FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ObjectID)));
}

/*
 * Time to string conversion
 */
CharPtr
TimeToString(ULong t, CharPtr s)
{
  Long min;
  Long hour;
  Long day;

  Char tmpStr[20];

  day = t / 86400;
  t %= 86400;
  hour = t / 3600;
  t %= 3600;
  min = t / 60;

  StrCopy(s, "");

  if (day > 0) {
	 StrIToA(tmpStr, day);
	 StrCat(s, tmpStr);
	 StrCat(s, ":");
  }

  if ((day > 0) && (hour < 10))
	 StrCat(s, "0");

  StrIToA(tmpStr, hour);
  StrCat(s, tmpStr);
  StrCat(s, ":");

  if (min < 10)
	 StrCat(s, "0");
  StrIToA(tmpStr, min);
  StrCat(s, tmpStr);

  return s;
}

/*
 * The Save button will stuff all time information as text into a
 * string of maximum length maxLen.
 */
void
SaveData(void)
{
  const Int maxLen = 4096; /* upper limit for memopad notes */
  UInt i;
  UInt choice = 0;
  Long sumTime = 0;
  Char clipText[maxLen];
  Char dateStr[longDateStrLength];
  Char timeStr[10];
  DateTimeType dateTime;
  Char textStr[MAXTEXT+1];
  VoidHand noteH;
  CharPtr noteP;

  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  DateToAscii(dateTime.month, dateTime.day, dateTime.year, dfDMYLongWithDot, dateStr);

  for(i = 0; i < MAXSLOT; i++)
	 sumTime += GetElapsed(i);

  StrCopy(clipText, "Titrax log ");
  StrCat(clipText, dateStr);
  StrCat(clipText, "\n");

  StrCat(clipText, "total time: ");
  StrCat(clipText, TimeToString(sumTime, timeStr));
  StrCat(clipText, "\n");

  StrCat(clipText, "project times:\n");

  for(i = 0; i < MAXSLOT; i++) {
	 if (TextIsEmpty(i) == false) {
		GetText(i, textStr);
		StrCat(clipText, textStr);
		StrCat(clipText, ": ");
		StrCat(clipText, TimeToString(GetElapsed(i), timeStr));
		StrCat(clipText, "\n");
	 }
  }

  StrCat(clipText, "Notes:\n");

  for(i = 0; i < MAXSLOT; i++) 
	 if (TextIsEmpty(i) == false) 
		if (NoteIsEmpty(i) == false) {
		  GetText(i, textStr);
		  if ((StrLen(clipText) + StrLen(textStr) + 1 + NoteLen(i) + 1) < (maxLen-1)) {
			 noteH = GetNote(i);
			 noteP = MemHandleLock(noteH);

			 StrCat(clipText, textStr);
			 StrCat(clipText, ":\n");
			 StrCat(clipText, noteP);
			 StrCat(clipText, "\n");

			 MemHandleUnlock(noteH);
			 MemHandleFree(noteH);
		  }
		  else {
			 choice = FrmAlert(titraxNoteSpaceAlert);
			 break;
		  }
		}
  
  if (choice == 0) 
	 CreateMemopadRecord(clipText);
}

void 
TimeString(CharPtr str)
{
  DateTimeType dateTime;

  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  TimeToAscii(dateTime.hour, dateTime.minute, tfColon24h, str);
}

void
DateString(CharPtr str)
{
  DateTimeType dateTime;

  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  DateToAscii(dateTime.month, dateTime.day, dateTime.year, dfDMYWithDots, str);
}

void
AddTimeStamp(Int no, Boolean start)
{
  Int len;
  VoidPtr strP;
  VoidHand strH;
  VoidPtr noteP;
  VoidHand noteH;
  Char timStr[timeStringLength];

  len = NoteLen(no);
  TimeString(timStr);

  strH = MemHandleNew(len + 7);
  strP = MemHandleLock(strH);

  noteH = GetNote(no);
  noteP = MemHandleLock(noteH);

  StrCopy(strP, noteP);

  MemHandleUnlock(noteH);
  MemHandleFree(noteH);

  StrCat(strP, timStr);
  if (start)
	 StrCat(strP, "-");
  else
	 StrCat(strP, "\n");

  SetNote(no, strP);

  MemHandleUnlock(strH);
  MemHandleFree(strH);
}

/*
 * Open the memopad database, create a new record and store a text in it
 */
Boolean
CreateMemopadRecord(CharPtr text)
{
  Int idx = 0;
  UInt mode;
  DmOpenRef MemopadDB;
  VoidPtr recP;
  VoidHand recH;

  mode = dmModeReadWrite;
  MemopadDB = DmOpenDatabaseByTypeCreator('DATA', 'memo', mode);

  if (!MemopadDB) {
	 (void)FrmAlert(titraxNotepadAlert);
	 return false;
  }

  recH = DmNewRecord(MemopadDB, &idx, StrLen(text)+1);
  recP = MemHandleLock(recH);

  if (DmSet(recP, 0, StrLen(text)+1, 0) != 0) {
	 (void)FrmAlert(titraxNotepadAlert);
	 return false;
  }

  DmWrite(recP, 0, text, StrLen(text));

  MemHandleUnlock(recH);
  DmReleaseRecord(MemopadDB, idx, true);

  DmCloseDatabase(MemopadDB);

  return true;
}
  
